@@ -1,7 +1,8 @@
-from typing import Callable
+from typing import Callable, List
 
 import numpy as np
 from pyPhases.util.Logger import classLogger
+from pyPhasesPreprocessing.Event import Event
 from pyPhasesRecordloader import ChannelsNotPresent, ParseError, RecordSignal, Signal
 
 
@@ -10,9 +11,7 @@ class Preprocessing:
     stepDefinition: dict = {}
     instance: "Preprocessing" = None
 
-    def addPreprocessingStep(
-        self, stepName: str, stepCallable: Callable[[Signal, RecordSignal, dict], None]
-    ):
+    def addPreprocessingStep(self, stepName: str, stepCallable: Callable[[Signal, RecordSignal, dict], None]):
         """Add a new preprocessing step to the preprocessing pipeline
 
         Args:
@@ -30,6 +29,7 @@ class Preprocessing:
         instance.targetChannels = preprocessingConfig["targetChannels"]
         instance.stepsByType = preprocessingConfig["stepsPerType"]
         instance.forceGapBetweenEvents = preprocessingConfig["forceGapBetweenEvents"]
+        instance.extendEvents = preprocessingConfig["extendEvents"] if "extendEvents" in preprocessingConfig else None
 
         cls.instance = instance
 
@@ -558,9 +558,7 @@ class Preprocessing:
     #     db.close()
     #     return quality
 
-    def parseSignalSteps(
-        self, signal: Signal, stepName, psgSignal: RecordSignal, channelConfig={}
-    ):
+    def parseSignalSteps(self, signal: Signal, stepName, psgSignal: RecordSignal, channelConfig={}):
         signal.processHistory.append(stepName)
         if stepName == "resampleFIR":
             signal.resample(psgSignal.targetFrequency, antialiaseFIR=True)
@@ -608,9 +606,7 @@ class Preprocessing:
             uniquePositions = set(np.unique(signal.signal))
             checkValues = set(uniquePositions) - set([0, 3, 6, 9, 12])
             if len(checkValues) > 0:
-                raise Exception(
-                    "alice position only supports 0, 3, 6, 9, 12 as values ... fix here :-)"
-                )
+                raise Exception("alice position only supports 0, 3, 6, 9, 12 as values ... fix here :-)")
 
             signal.signal[signal.signal == 0] = 1
             signal.signal[signal.signal == 3] = 5
@@ -621,9 +617,7 @@ class Preprocessing:
             uniquePositions = set(np.unique(signal.signal))
             checkValues = set(uniquePositions) - set([1, 2, 3, 4, 5, 6])
             if len(checkValues) > 0:
-                raise Exception(
-                    "domino position only supports 1, 2, 3, 4, 5, 6 as values ... fix here :-)"
-                )
+                raise Exception("domino position only supports 1, 2, 3, 4, 5, 6 as values ... fix here :-)")
 
             signal.signal[signal.signal == 1] = 4
             signal.signal[signal.signal == 2] = 1
@@ -637,8 +631,11 @@ class Preprocessing:
             # RIGHT, LEFT, BACK, FRONT (derived from the profusion xml, not sure if the mapping is actually correct)
             checkValues = set(uniquePositions) - set([0, 1, 2, 3])
             if len(checkValues) > 0:
-                raise Exception(
-                    "domino position only supports 0, 1, 2, 3 as values ... fix here :-)"
+                # there are some records with invalid values (like shhs1-202947), we just set them to 0
+                # shhs1-203716
+                signal.signal[np.isin(signal.signal, list(checkValues))] = 0
+                self.logError(
+                    "shhs position only supports 0, 1, 2, 3 as values, conflicts: %s \n... fix here :-)" % checkValues
                 )
 
             signal.signal += 10  # overwrite protection
@@ -651,9 +648,7 @@ class Preprocessing:
             # Right, Back, Left, Front, Upright (derived from the profusion xml, not sure if the mapping is actually correct)
             checkValues = set(uniquePositions) - set([0, 1, 2, 3, 4])
             if len(checkValues) > 0:
-                raise Exception(
-                    "domino position only supports 0, 1, 2, 3, 4 as values ... fix here :-)"
-                )
+                raise Exception("domino position only supports 0, 1, 2, 3, 4 as values ... fix here :-)")
 
             signal.signal += 10  # overwrite protection
             signal.signal[signal.signal == 10] = 5
@@ -664,22 +659,16 @@ class Preprocessing:
         elif stepName == "rr2hr":
             timeseries = TimeseriesSignal(signal)
             timeseries.rr2hr()
-            signal.signal = timeseries.resampleAtFrequency(
-                signal.signal.shape[0], signal.frequency
-            )
+            signal.signal = timeseries.resampleAtFrequency(signal.signal.shape[0], signal.frequency)
         elif stepName in Preprocessing.stepDefinition:
             Preprocessing.stepDefinition[stepName](signal, psgSignal, channelConfig)
         else:
-            raise Exception(
-                "The Preprocessing step '%s' is not yet supported" % stepName
-            )
+            raise Exception("The Preprocessing step '%s' is not yet supported" % stepName)
 
     def preprocessingSignal(self, psgSignal: RecordSignal):
         self.preprocessingSignal(psgSignal, self.stepsByType)
 
-    def preprocessSignalByType(
-        self, psgSignal: RecordSignal, stepsByType, targetFrequency
-    ):
+    def preprocessSignalByType(self, psgSignal: RecordSignal, stepsByType, targetFrequency):
         psgSignal.targetFrequency = targetFrequency
         for signal in psgSignal.signals:
             cName = signal.name
@@ -699,11 +688,23 @@ class Preprocessing:
                     )
             else:
                 if cName not in self.optionalSignals and not signal["generated"]:
-                    self.logError(
-                        "Missing channel %s for %s" % (cName, signal.recordId)
-                    )
+                    self.logError("Missing channel %s for %s" % (cName, signal.recordId))
                     raise ChannelsNotPresent(cName, signal.recordId)
 
+    def preprocessEvents(self, events: List[Event]):
+        if self.extendEvents is not None:
+            events = self.preprocessEventsByConfig(events, self.extendEvents)
+        return events
+
+    def preprocessEventsByConfig(self, events: List[Event], extendEventsConfig):
+        extendEventMap = extendEventsConfig
+        for event in events:
+            if event.name in extendEventMap:
+                addBefore, addAfter = extendEventMap[event.name]
+                event.start -= addBefore
+                event.duration += addAfter + addAfter
+        return events
+
     # def extractRecordAndAnnotations(self, recordName):
     #     return self.extractRecord(recordName, True)
 
